﻿ /**
    ■ガイドによりスライスするスクリプト　v210810
　▼構造前提条件
        最上段はレイヤーセットを親としている構成である
        カラーモードはRGBのみ対応
  */
 
#include "Lib_generalMethod.jsx";
//ドキュメントの各種定義
var docObj = activeDocument;//ドキュメント
docObj.activeLayer = docObj.layers[0];//active化
const defStr = {
    errorGuide: "ガイドが設定されていません。\nガイドを設定してください。",
    errorGuideX:"縦ラインのガイドは2つ設定してください。",
    errorGuideXOutSide:"縦ラインのガイドが画像の外側に設定されています",
    errorGuideY:"横ラインのガイドは2つ設定してください。",
    errorGuideYOutSide:"横ラインのガイドが画像の外側に設定されています",
    layerSetName:"■sliceObj",    
};

//変数定義
var lineSlice = 0;
var sliceArea;
var bound;
var size={
    startX:0,
    startY:0,
    endX:0,
    endY:0,
};

app.activeDocument.suspendHistory("script：Act_docResize_multiple", "Act_sliceTexture()");
function Act_sliceTexture(){
	try{
        var myRulerUnits = preferences.rulerUnits;
        preferences.rulerUnits = Units.PIXELS;
        
        //素材の全サイズを取得し
        Act_CreateSmartObj();//スマートオブジェクト化
        bound = new GetBounds();
        CanvasCrop(bound.startX,bound.startY,bound.endX ,bound.endY);
        
        var guidesX = new GetGuides(0);
        var guidesY = new GetGuides(1);
        new GuideCheck(guidesX,guidesY);
        sliceArea = new SetSliceArea(guidesX, guidesY) ;
        bound = new GetBounds;
        if(lineSlice == 2){
            SetSelection(bound.startX,bound.startY,sliceArea.startX,bound.endY);
            LayerDuplicate();
            SetSelection(sliceArea.endX,bound.startY,bound.endX,bound.endY); 
            TranslateLayer((sliceArea.endX - sliceArea.startX)*-1,"x");
            size.endX = bound.endX - sliceArea.endX + sliceArea.startX;
            size.endY = bound.endY;
        }else if(lineSlice == 1){
            SetSelection(bound.startX,bound.startY,bound.endX,sliceArea.startY);
            LayerDuplicate();
            SetSelection(bound.startX,sliceArea.endY,bound.endX,bound.endY); 
            TranslateLayer((sliceArea.endY - sliceArea.startY)*-1,"y");
            size.endX = bound.endX;
            size.endY = bound.endY - sliceArea.endY + sliceArea.startY
        }else{
            SetSelection(bound.startX,bound.startY,sliceArea.startX,sliceArea.startY); 
            LayerDuplicate();
            SetSelection(sliceArea.endX,bound.startY,bound.endX,sliceArea.startY); 
            LayerDuplicate();
            SetSelection(bound.startX,sliceArea.endY,sliceArea.startX,bound.endY); 
            LayerDuplicate();
            SetSelection(sliceArea.endX,sliceArea.endY,bound.endX,bound.endY); 
            size.endX = bound.endX - sliceArea.endX + sliceArea.startX;
            size.endY = bound.endY - sliceArea.endY + sliceArea.startY;
        };
        //レイヤーセットにまとめる
        Act_CreateLayerSet(defStr.layerSetName);
        if(lineSlice == 0){
            for(var i = 4;i > 0;i--){
                docObj.activeLayer = docObj.layers[i];//active化
                if(i == 1){
                    TranslateLayer((sliceArea.endX - sliceArea.startX)*-1,"x");
                    TranslateLayer((sliceArea.endY - sliceArea.startY)*-1,"y");
                    Act_MoveInsideLayerSet(defStr.layerSetName);
                };
                if(i == 2){
                    TranslateLayer((sliceArea.endY - sliceArea.startY)*-1,"y");
                    Act_MoveInsideLayerSet(defStr.layerSetName);
                };
                if(i == 3){
                    TranslateLayer((sliceArea.endX - sliceArea.startX)*-1,"x");
                    Act_MoveInsideLayerSet(defStr.layerSetName);
                };
                if(i == 4){
                    Act_MoveInsideLayerSet(defStr.layerSetName);
                };
            };
        };
        if(lineSlice == 1){
            for(var i = 2;i > 0;i--){
                docObj.activeLayer = docObj.layers[i];//active化
                if(i == 1){
                    TranslateLayer((sliceArea.endX - sliceArea.startX)*-1,"x");
                    Act_MoveInsideLayerSet(defStr.layerSetName);
                };
                if(i == 2){
                    Act_MoveInsideLayerSet(defStr.layerSetName);
                };
            };
        };
        if(lineSlice == 2){
            for(var i = 2;i > 0;i--){
                docObj.activeLayer = docObj.layers[i];//active化
                if(i == 1){
                    TranslateLayer((sliceArea.endY - sliceArea.startY)*-1,"y");
                    Act_MoveInsideLayerSet(defStr.layerSetName);
                };
                if(i == 2){
                    Act_MoveInsideLayerSet(defStr.layerSetName);
                };
            };
        };    
        CanvasCrop(size.startX,size.startY,size.endX ,size.endY);
	}catch(e) {
		alert("error("+e+")");
	};
    preferences.rulerUnits = myRulerUnits;
};

//関数：ガイドの確認
function GuideCheck(guidesX, guidesY) {
    // ガイドが設定されているか？
    if (app.activeDocument.guides.length == 0) {
        throw defStr.errorGuide;
    }
    // 縦ラインガイドのチェック
    if (guidesX.length == 0){
        lineSlice = 1;
    }else if (guidesX.length != 2  && guidesX.length != 0) {
        throw defStr.errorGuideX;
    }else if (guidesX[0].coordinate.value < guidesX[1].coordinate.value) {
        if(guidesX[0].coordinate.value <= 0 || guidesX[1].coordinate.value >= bound.sizeX){
                throw defStr.errorGuideXOutSide;
        };
    } else {
        if(guidesX[1].coordinate.value <= 0 || guidesX[0].coordinate.value >= bound.sizeX){
                throw defStr.errorGuideXOutSide;
        };
    };
    // 横ラインガイドのチェック
    if (guidesY.length == 0){
        lineSlice = 2;
    }else if (guidesY.length != 2 && guidesY.length != 0){
            throw defStr.errorGuideY;
    }else if (guidesY[0].coordinate.value < guidesY[1].coordinate.value) {
        if(guidesY[0].coordinate.value <= 0 || guidesY[1].coordinate.value >= bound.sizeY){
                throw defStr.errorGuideYOutSide;
        };
    } else {
        if(guidesY[1].coordinate.value <= 0 || guidesY[0].coordinate.value >= bound.sizeY){
                throw defStr.errorGuideYOutSide;
        };
    };
};
//関数：ガイド取得
function GetGuides(num) {
    var guide = app.activeDocument.guides;
    var guides = [];
    var guidesNum = [];
    var equal =false;
    var first =true;
    for (var i =0; i < guide.length; i++) {
        equal = false;
        if (num == 0){
           if (guide[i].direction == Direction.VERTICAL) {
                if(first){
                    guidesNum.push(guide[i].coordinate.value);
                    guides.push(guide[i]);
                    first = false;
                }else{    
                    for (var j in guidesNum) {
                        if(guidesNum[j] == guide[i].coordinate.value){
                            equal = true;break;
                        };
                    };
                    if(!equal){
                        guides.push(guide[i]);
                        guidesNum.push(guide[i].coordinate.value);
                    };
                };
            };
        }else if(num > 0){
            if (guide[i].direction == Direction.HORIZONTAL) {
                if(first){
                    guidesNum.push(guide[i].coordinate.value);
                    guides.push(guide[i]);
                    first = false;
                }else{  
                    for (var j in guidesNum) {
                        if(guidesNum[j] == guide[i].coordinate.value){
                            equal = true;break;
                        };
                    };
                    if(!equal){
                        guides.push(guide[i]);
                        guidesNum.push(guide[i].coordinate.value);
                    };
                };
            };
        };
    };
    return guides;
};
//関数：スライスエリア取得
 function SetSliceArea(guidesX, guidesY) {
    var sliceArea = new SliceArea (0, 0, 0, 0);
    if (guidesX.length != 0) {
        var valueA = guidesX[0].coordinate.value;
        var valueB = guidesX[1].coordinate.value;
        
        if (valueA > valueB) {
            sliceArea.startX = valueB;
            sliceArea.endX = valueA;
        } else {
            sliceArea.startX = valueA;
            sliceArea.endX = valueB;
        };
    };
    if (guidesY.length != 0) {
        var valueA = guidesY[0].coordinate.value;
        var valueB = guidesY[1].coordinate.value;

        if (valueA > valueB) {
            sliceArea.startY = valueB;
            sliceArea.endY = valueA;
        } else {
            sliceArea.startY = valueA;
            sliceArea.endY = valueB;
        };
    };
    return sliceArea;
};
//関数：スライスエリアの初期化
 function SliceArea(startX, startY, endX, endY) {
		this.startX = startX;
		this.startY = startY;
		this.endX = endX;
		this.endY = endY;
};
//関数：親レイヤーのピクセル範囲を取得
function GetBounds(){
    var bound={
    sizeX:0,
    sizeY:0,
    startX:0,
    startY:0,
    endX:0,
    endY:0,
    };
    layBounds = docObj.layers[0].bounds;
    layBounds = docObj.layers[0].bounds;
    bound.startX = parseInt(layBounds[0]);
    bound.startY = parseInt(layBounds[1]);
    bound.endX  = parseInt(layBounds[2]);
    bound.endY  = parseInt(layBounds[3]);
    bound.sizeX = (bound.endX - bound.startX);
    bound.sizeY = (bound.endY - bound.startY);
    return bound;
};
//関数：カンバスのクロップ
function CanvasCrop(startX,startY,endX,endY){
    docObj.crop([startX,startY,endX ,endY]);
};
//関数：選択範囲の生成
function SetSelection(pX1,pY2,pX3,pY4){
    var region = new Array();
    region[0] = [pX1, pY2];
    region[1] = [pX1, pY4];
    region[2] = [pX3, pY4];
    region[3] = [pX3, pY2];    

    docObj.selection.select(region, SelectionType.REPLACE, 0, false);
    Act_CreateLayerMask();
    var cObj= new SolidColor();
    cObj.rgb.red= 255;
    cObj.rgb.green= 255;
    cObj.rgb.blue= 255;
    app.foregroundColor= cObj;
    cObj.rgb.red=0;
    cObj.rgb.green= 0;
    cObj.rgb.blue= 0;
    app.backgroundColor= cObj;
    LayerMaskFill();
    docObj.selection.deselect();
};
//関数：レイヤーマスク塗りつぶし
function LayerMaskFill(){
    var idFl = charIDToTypeID( "Fl  " );
        var desc1 = new ActionDescriptor();
        var idUsng = charIDToTypeID( "Usng" );
        var idFlCn = charIDToTypeID( "FlCn" );
        var idFrgC = charIDToTypeID( "FrgC" );
        desc1.putEnumerated( idUsng, idFlCn, idFrgC );
        var idOpct = charIDToTypeID( "Opct" );
        var idPrc = charIDToTypeID( "#Prc" );
        desc1.putUnitDouble( idOpct, idPrc, 100.000000 );
        var idMd = charIDToTypeID( "Md  " );
        var idBlnM = charIDToTypeID( "BlnM" );
        var idNrml = charIDToTypeID( "Nrml" );
        desc1.putEnumerated( idMd, idBlnM, idNrml );
    executeAction( idFl, desc1, DialogModes.NO );
};
//関数：レイヤー複製しマスクを初期化
function LayerDuplicate(){
    docObj.activeLayer.duplicate();
    docObj.activeLayer = docObj.layers[0];//active化
    HasLayerMaskDelete();
};
//関数：レイヤーマスクを保持していれば削除
function HasLayerMaskDelete(){
if(activeDocument.activeLayer.typename == "ArtLayer" && HasLayerMask() == true){
    var idDlt = charIDToTypeID( "Dlt " );
    var desc1 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref1 = new ActionReference();
    var idChnl = charIDToTypeID( "Chnl" );
    var idChnl = charIDToTypeID( "Chnl" );
    var idMsk = charIDToTypeID( "Msk " );
    ref1.putEnumerated( idChnl, idChnl, idMsk );
    desc1.putReference( idnull, ref1 );
    executeAction( idDlt, desc1, DialogModes.NO );
    };
};
//関数：レイヤーマスクが保持されているか判定
function HasLayerMask() {
    var hasLayerMask = false;
    try {
        var ref = new ActionReference();
        var keyUserMaskEnabled = app.charIDToTypeID( 'UsrM' );
        ref.putProperty( app.charIDToTypeID( 'Prpr' ), keyUserMaskEnabled );
        ref.putEnumerated( app.charIDToTypeID( 'Lyr ' ), app.charIDToTypeID( 'Ordn' ), app.charIDToTypeID( 'Trgt' ) );
        var desc = executeActionGet( ref );
        if ( desc.hasKey( keyUserMaskEnabled ) ) {
            hasLayerMask = true;
        };
    }catch(e) {
        hasLayerMask = false;
        }
    return hasLayerMask;
};
//関数：レイヤーの移動
function TranslateLayer(num,direction) {
    if(direction == "x"){
        docObj.activeLayer.translate(num , 0);
    }else{
        docObj.activeLayer.translate(0 , num);
    };
};